import os
import sys
import shutil
from matplotlib import get_configdir
from time import time
from . import styles
from .constants import DECIMALS
from .parameters import journals
from .figures import standard_figsize, standard_rect


def create_matplotlib_style(journal="PR", folder=""):
    """Creates a matplotlib style file in a given folder"""
    d = journals[journal]
    figsz = standard_figsize(journal)
    rect = standard_rect(journal)
    if not os.path.exists(folder):
        os.makedirs(folder)
    with open(folder + "/{}.mplstyle".format(journal), "w") as f:
        f.write("figure.figsize : {},{}\n".format(*figsz))
        f.write("figure.subplot.left : {}\n".format(rect[0]))
        f.write("figure.subplot.bottom : {}\n".format(rect[1]))
        f.write(
            "figure.subplot.right : {}\n".format(round(rect[0] + rect[2], DECIMALS))
        )
        f.write("figure.subplot.top : {}\n".format(round(rect[1] + rect[3], DECIMALS)))
        try:
            for par in d["rcparameters"]:
                f.write("{} : {}\n".format(par, d["rcparameters"][par]))
        except Exception:
            pass


def diff_dic(dic1, dic2):
    """returns the dfference of two dictionaries as sorted set"""
    mismatch = sorted({key for key in dic1.keys() & dic2 if dic1[key] != dic2[key]})
    return mismatch


if __name__ == "__main__":
    sdir = styles.__path__[0]  # gets styles directory
    cdir = get_configdir()  # gets matplotlib config directory
    if sys.argv[1] == "--help" or sys.argv[1] == "-h":
        print(
            """This script installs configuration styles in matplotlib config directory.
              
To remove configuration: 
1) Go to {cdir}, delete matplotlibrc file and reinstate the previous matplotlibrc file.
Files that have been substituted have been backupped by appending _backup_<time>.  
2) Go to {cdir}//stylelib and delete the undesired .mplstyle files.
The files created by this package are: 
{mplfiles}
3) pip uninstall journal-styles
              
Options:
    install     Installs matplolib styles in matplotlib config directory. Old files are backupped.
    update-styles   Updates the stylefiles in the internal package directory and copies them to 
                    the matplotlib config directory.""".format(
                cdir=cdir,
                mplfiles="".join(
                    ["\n" + f for f in os.listdir(sdir) if f[-8:] == "mplstyle"]
                ),
            )
        )

    elif sys.argv[1] == "install":
        # backup matplotlibrc if present
        if os.path.exists(cdir + "//matplotlibrc"):
            shutil.copy(
                cdir + "//matplotlibrc",
                cdir + "//matplotlibrc_backup_{}".format(time()),
            )
        # substitute matplotlibrc
        shutil.copy(sdir + "//matplotlibrc", cdir + "//matplotlibrc")
        # update styles in local folder
        for journal in journals:
            create_matplotlib_style(journal=journal, folder=sdir)
        # create styles directory if neeeded
        if not os.path.exists(cdir + "//stylelib"):
            os.makedirs(cdir + "//stylelib")
        # copy all the mplstyle into matploltib config directory backupping old files
        for f in os.listdir(sdir):
            if f[-8:] == "mplstyle":
                if os.path.exists(cdir + "//stylelib//{}".format(f)):
                    shutil.copy(
                        cdir + "//stylelib//{}".format(f),
                        cdir + "//stylelib//{}".format(f) + "_backup_{}".format(time()),
                    )
                shutil.copy(sdir + "//" + f, cdir + "//stylelib//{}".format(f))
        # line = "c.InlineBackend.rc = { }\n"
        # config_file = "{}/.ipython/profile_default/ipython_kernel_config.py".format(str(Path.home()))

    elif sys.argv[1] == "update-styles":
        # update styles in local folder
        for journal in journals:
            create_matplotlib_style(journal=journal, folder=sdir)
        # create styles directory if neeeded
        if not os.path.exists(cdir + "//stylelib"):
            os.makedirs(cdir + "//stylelib")
        # copy all the mplstyle into matploltib config directory backupping old files
        for f in os.listdir(sdir):
            if f[-8:] == "mplstyle":
                if os.path.exists(cdir + "//stylelib//{}".format(f)):
                    shutil.copy(
                        cdir + "//stylelib//{}".format(f),
                        cdir + "//stylelib//{}".format(f) + "_backup_{}".format(time()),
                    )
                shutil.copy(sdir + "//" + f, cdir + "//stylelib//{}".format(f))

    else:
        print("unkwnon option {}".format(sys.argv[1]))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from .parameters import journals
from .constants import golden_ratio, DECIMALS


def standard_figsize(journal="PR", aspect_ratio=golden_ratio):
    """returns the standard figsize for a given journal (1-D plot)
    example of usage
    fs = standard_figsize('PR')
    plt.figure(figsize = fs)"""
    d = journals[journal]
    w = d["onecolumn"]
    h = w / aspect_ratio
    return (round(w, DECIMALS), round(h, DECIMALS))


def standard_rect(journal="PR", aspect_ratio=golden_ratio):
    """returns the standard axes rectangle for a given journal (1-D plot)
    example of usage
    as = standard_rect('PR')
    plt.axes(rect = as)"""
    d = journals[journal]
    w = d["onecolumn"]
    h = w / aspect_ratio
    return [
        round(d["hspace_l"] / w, DECIMALS),
        round(d["vspace_l"] / h, DECIMALS),
        round(1.0 - (d["hspace_s"] + d["hspace_l"]) / w, DECIMALS),
        round(1.0 - (d["vspace_s"] + d["vspace_l"]) / h, DECIMALS),
    ]


class Figure_Frame:
    """Class for composing non-standard figures.
    Example of usage :
    ff = figure_frame('lMs','lMs')
    plt.figure(figsize = ff.figsize)
    plt.axes(ff.rects[0])

    methods
    -------
    __init__

    attributes
    ----------
    figsize (tuple) : figure width and height in inches
    rects (list) : each element is a list of type [left, bottom, width, height]
    representinx axes position expressed in units of figure width and height.
    """

    def __init__(
        self,
        h_spacing="lMs",
        v_spacing="lMs",
        aspect_ratio=golden_ratio,
        journal="PR",
        column="onecolumn",
        additional_axes=[],
    ):
        """
        parameters
        -----------
        h_spacing : string, horizontal division of the figure (left to right).
            Can contain 's' (small space), 'l' (large space), 'h' (half small space),
            'H' (half large space). Must contain 'M' (position of the main axes)
        v_spacing : string, vertical division of the figure (bottom to top).
        aspect_ratio : float, aspect ratio (width/heigth) of the main axes
        journal : string, name of an available journal
        column : string or float, name of an available column type (journal dependent)
            or float (column width in inches)
        additional_axes : list of (int, int) tuples, integer coordinates of the
            rectangle corresponding to secondary axes
        ---------
        returns
        ---------
        instance of the class
        """
        d = journals[journal]
        if isinstance(column, float):
            fig_width = column
        else:
            fig_width = d[column]
        dic_h = {
            "s": d["hspace_s"],
            "l": d["hspace_l"],
            "h": d["hspace_s"] / 2.0,
            "H": d["hspace_l"] / 2.0,
            "M": 0.0,
        }
        dic_v = {
            "s": d["vspace_s"],
            "l": d["vspace_l"],
            "h": d["vspace_s"] / 2.0,
            "H": d["vspace_l"] / 2.0,
            "M": 0.0,
        }
        diff = sum([dic_h[x] for x in h_spacing])
        dic_h["M"] = (fig_width - diff) / h_spacing.count("M")
        dic_v["M"] = dic_h["M"] / aspect_ratio
        fig_height = sum([dic_v[x] for x in v_spacing])
        figsize = (round(fig_width, DECIMALS), round(fig_height, DECIMALS))
        h_indices = [i for i, a in enumerate(h_spacing) if a == "M"]
        v_indices = [i for i, a in enumerate(v_spacing) if a == "M"]
        axes = [(i, j) for j in v_indices for i in h_indices] + additional_axes
        # [(h_spacing.index('M'), v_spacing.index('M'))]

        rects = []
        for ax in axes:
            x0 = sum([dic_h[h_spacing[i]] for i in range(ax[0])])
            y0 = sum([dic_v[v_spacing[i]] for i in range(ax[1])])
            rects.append(
                [
                    round(x0 / fig_width, DECIMALS),
                    round(y0 / fig_height, DECIMALS),
                    round(dic_h[h_spacing[ax[0]]] / fig_width, DECIMALS),
                    round(dic_v[v_spacing[ax[1]]] / fig_height, DECIMALS),
                ]
            )
        self.figsize = figsize
        self.rects = rects



# pt is LaTeX point = 1/72.27 inch
inch2pt = 72.27
pt2inch = 1.0 / inch2pt
inch2mm = 25.4
mm2inch = 1.0 / inch2mm
pt2mm = pt2inch * inch2mm  # 0.35145980
mm2pt = 1.0 / pt2mm
golden_ratio = 1.6180
DECIMALS = 3  # decimal precision



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Module providing figure styles and dimension for scientific journals"""

__version__ = "1.1.0"
__author__ = "Iacopo Torre"
# from .figures import standard_figsize as standard_figsize
# from .figures import standard_rect as standard_rect
from .figures import Figure_Frame as Figure_Frame
from .figures import Figure_Frame as figure_frame
from .parameters import journals as journals



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from .constants import golden_ratio, pt2inch, mm2inch

# Physical Review family
parameters_PR = {
    "onecolumn": 246 * pt2inch,
    "twocolumn": 510 * pt2inch,
    "hspace_l": 246 * pt2inch * 0.18,
    "hspace_s": 246 * pt2inch * 0.045,
    "vspace_l": 246 * pt2inch / golden_ratio * 0.18,
    "vspace_s": 246 * pt2inch / golden_ratio * 0.045,
    #'font' : 'Computer Modern Roman',
    "rcparameters": {
        "text.usetex": True,
        "savefig.dpi": 300,
        "font.size": 10,
        "font.family": "serif",
    },
}
# Nature
parameters_nature = {
    "onecolumn": 89 * mm2inch,
    "onehalfcolumn120": 120 * mm2inch,
    "onehalfcolumn136": 136 * mm2inch,
    "twocolumn": 183 * mm2inch,
    "hspace_l": 14 * mm2inch,
    "hspace_s": 3.5 * mm2inch,
    "vspace_l": 14 * mm2inch / golden_ratio,
    "vspace_s": 3.5 * mm2inch / golden_ratio,
    #'font' : 'Arial',
    "rcparameters": {
        "text.usetex": False,
        "savefig.dpi": 300,
        "font.size": 7,
        "font.family": "sans-serif",
    },
}
# Nature Communications
parameters_nature_comm = {
    "onecolumn": 88 * mm2inch,
    "twocolumn": 180 * mm2inch,
    "hspace_l": 14 * mm2inch,
    "hspace_s": 3.5 * mm2inch,
    "vspace_l": 14 * mm2inch / golden_ratio,
    "vspace_s": 3.5 * mm2inch / golden_ratio,
    #'font' : 'Arial',
    "rcparameters": {
        "text.usetex": False,
        "savefig.dpi": 300,
        "font.size": 7,
        "font.family": "sans-serif",
    },
}
# Science
parameters_science = {
    "onecolumn": 55 * mm2inch,
    "twocolumn": 120 * mm2inch,
    "hspace_l": 14 * mm2inch,
    "hspace_s": 3.5 * mm2inch,
    "vspace_l": 14 * mm2inch / golden_ratio,
    "vspace_s": 3.5 * mm2inch / golden_ratio,
    #'font' : 'Helvetica',
    "rcparameters": {
        "text.usetex": False,
        "savefig.dpi": 300,
        "font.size": 7,
        "font.family": "sans-serif",
    },
}
# add new journal here
######################### Available journals ###################################
# update dictionary when a new journal is added
journals = {
    "PR": parameters_PR,
    "Nature": parameters_nature,
    "Science": parameters_science,
    "Nature_Comm": parameters_nature_comm,
}
################################################################################
